# This is a script to make the alignment of multiple datasets generated by the data processing module of IsoMS.
# Tao Huan, Feb 05, 2014
# Copyright @ 2014 University of Alberta

#################################################################################
# This is the setting part
data.path <- "I:/20131022UA_Opthalmology_IsoMS/pairs/"
Alignment.level <- 2 
# "1" to align level 1, "2" to align level 1 and 2, "3" to align level 1, 2, and 3.
rt.tol <- 10 # default retention time tolerance is 10 seconds
mz.tol <- 8 # default mz tolerance is 8 ppm
##################################################################################
setwd(data.path)
data.file <- list.files(pattern = ".csv")
flag <- 1
while (is.na(data.file[flag]) == FALSE)
  { 
  print(paste("Processing", "file", flag))
  file <- read.csv(data.file[flag])
  if (Alignment.level == 1){
    file <- file[file$pairLev == 1,]
  } else if (Alignment.level == 2){
    file <- file[(file$pairLev == 1) | (file$pairLev == 2),]
  } else if (Alignment.level == 3){
    file <- file[(file$pairLev == 1) | (file$pairLev == 2) | (file$pairLev == 3),]
  }
  if (flag == 1){
    file.master <- cbind(file[,1:9], file$ratio_light_to_heavy)
    names(file.master)[ncol(file.master)] <- data.file[flag]
  } else {
    file.master <- cbind(file.master, file.master$sample)
    file.extra <- file.master[1,]
    names(file.master)[ncol(file.master)] <- data.file[flag]
    file.master[,ncol(file.master)] <- NA
    i = 1
    while (is.na(file[i,1]) == FALSE){
      feature <- file[i,]
      mz.diff <- 1e6 * abs(file.master$mz_light - feature$mz_light) / feature$mz_light
      rt.diff <- abs(file.master$rt - feature$rt)
      int.diff <- abs(log10(file.master$int_light / feature$int_light))
      score <- (1 - mz.diff / mz.tol) + (1 - rt.diff / rt.tol) + (1 - int.diff)
      score[is.na(score)] <- -100
      if (max(score) > 1){
        file.master[score == max(score), ncol(file.master)] <- feature$ratio_light_to_heavy
        file.master[score == max(score), 1:9] <- (file.master[score == max(score), 1:9] + feature[1:9]) / 2 
      } else {
        feature.E <- as.data.frame(c(feature[1:9], rep(NA, flag-1), feature$ratio_light_to_heavy))
        colnames(feature.E) <- colnames(file.extra)
        if (is.na(feature.E[nrow(feature.E)]) == FALSE){
          file.extra <- rbind(file.extra, feature.E)
        }
      }
      i = i + 1
    }
    colnames(file.extra) <- colnames(file.master)
    if (nrow(file.extra) > 1){
      file.master <- rbind(file.master, file.extra[2:nrow(file.extra),])
    }
    rm(file.extra)
  }
  file.master <- file.master[order(file.master$rt),]
  flag <- flag + 1
}
row.names(file.master) <- c(1:nrow(file.master))
file.master[is.na(file.master)] <- " "
file.master <- cbind(file.master[,3:8], file.master[,10:ncol(file.master)])
print(paste("In total,", flag-1, "files have been processed. Done everything, Please check the Alignment_result.csv in the data folder"))
write.csv(file.master, file = "Alignment_result.csv")